/* Copyright (c) 2015 Nordic Semiconductor. All Rights Reserved.
 *
 * The information contained herein is property of Nordic Semiconductor ASA.
 * Terms and conditions of usage are described in detail in NORDIC
 * SEMICONDUCTOR STANDARD SOFTWARE LICENSE AGREEMENT.
 *
 * Licensees are granted free, non-transferable use of the information. NO
 * WARRANTY of ANY KIND is provided. This heading must NOT be removed from
 * the file.
 *
 */

/**
 * @file
 * @brief File with example code presenting usage of drivers for TWIS slave and TWI in master mode
 *
 * @sa twi_master_with_twis_slave_example
 */

/**
 * @defgroup twi_master_with_twis_slave_example Example code presenting usage of drivers for TWIS slave and TWI in master mode
 *
 * This code presents usage of the two drivers:
 * - @ref nrf_twi_drv (in synchronous mode)
 * - @ref nrf_twis_drv (in asynchronous mode)
 *
 * On the slave an EEPROM memory is simulated.
 * For simulated EEPROM AT24C01C device produced by ATMEL was selected.
 * This device has 128 bytes of memory and it is simulated using internal RAM.
 * This RAM area is accessed only by simulated EEPROM so the rest of application can access it
 * only using TWI commands via hardware configured pins.
 *
 * Selected memory chip uses 7 bits constant address. Word to access is selected during
 * write operation: first byte send is used as current address pointer.
 *
 * Maximum 8 byte page can be written in single access.
 * The whole memory can be read in single access.
 *
 * Differences between real chip and simulated one:
 * 1. Simulated chip has practically 0ms write time.
 *    This example does not poll the memory for readiness.
 * 2. During sequential read, when memory end is reached, zeroes are send.
 *    There is no support for roll-over.
 * 3. It is possible to write maximum of 8 bytes in single sequential write.
 *    But in simulated EEPROM the whole address pointer is incremented.
 *    In real memory chip only 3 lowest bits changes during writing.
 *    In real device writing would roll-over in memory page.
 *
 * On the master side we communicate with that memory and allow write and read.
 * We can use simple commands via UART to check the memory.
 *
 * Pins to short:
 * - @ref TWI_SCL_M - @ref EEPROM_SIM_SCL_S
 * - @ref TWI_SDA_M - @ref EEPROM_SIM_SDA_S
 *
 * Supported commands are always listed in the welcome message.
 * @{
 */
#include <stdbool.h>
#include <stdint.h>
#include <stdio.h>
#include <ctype.h>
#include "config.h"
#include "eeprom_simulator.h"
#include "app_uart.h"
#include "nrf_drv_twi.h"
#include "nrf_gpio.h"
#include "app_error.h"
#include "nrf.h"
#include "bsp.h"
#include "app_util_platform.h"

#include "boards.h"
#include "nrf_delay.h"
#include "mpu9150.h"
#include "mpu9150_register_map.h"


/*Pins to connect MPU. */
#define MPU9150_TWI_SCL_PIN 3//1
#define MPU9150_TWI_SDA_PIN 2

/*UART buffer size. */
//#define UART_TX_BUF_SIZE 256
//#define UART_RX_BUF_SIZE 1
static const nrf_drv_twi_t m_twi_instance = NRF_DRV_TWI_INSTANCE(0);




/**
 * @brief Handle UART errors
 *
 * Simple function for handling any error from UART module.
 * See UART example for more information.
 *
 * @param[in] p_event Event structure
 */
static void uart_error_handle(app_uart_evt_t * p_event)
{
    if (p_event->evt_type == APP_UART_COMMUNICATION_ERROR)
    {
        APP_ERROR_HANDLER(p_event->data.error_communication);
    }
    else if (p_event->evt_type == APP_UART_FIFO_ERROR)
    {
        APP_ERROR_HANDLER(p_event->data.error_code);
    }
}
//MPU9150\[X
/**
 * @brief UART events handler.
 */
static void uart_events_handler(app_uart_evt_t * p_event)
{
    switch (p_event->evt_type)
    {
        case APP_UART_COMMUNICATION_ERROR:
            APP_ERROR_HANDLER(p_event->data.error_communication);
            break;

        case APP_UART_FIFO_ERROR:
            APP_ERROR_HANDLER(p_event->data.error_code);
            break;

        default:
            break;
    }
}

/**
 * @brief TWI events handler.
 */
void twi_handler(nrf_drv_twi_evt_t const * p_event, void * p_context)
{   
    // Pass TWI events down to the MPU driver.
    mpu9150_twi_event_handler(p_event);
}

/**
 * @brief TWI initialization.
 * Just the usual way. Nothing special here
 */
void twi_init(void)
{
    ret_code_t err_code;
    
    const nrf_drv_twi_config_t twi_mpu_9150_config = {
       .scl                = MPU9150_TWI_SCL_PIN,
       .sda                = MPU9150_TWI_SDA_PIN,
       .frequency          = NRF_TWI_FREQ_400K,
       .interrupt_priority = APP_IRQ_PRIORITY_HIGH
    };
    
		err_code =    nrf_drv_twi_init(&m_twi_instance, &twi_mpu_9150_config, NULL);
		//err_code = nrf_drv_twi_init(&m_twi_instance, &twi_mpu_9150_config, twi_handler, NULL);
    APP_ERROR_CHECK(err_code);
    
    nrf_drv_twi_enable(&m_twi_instance);
}


void mpu_init(void)
{
    ret_code_t ret_code;
    // Initiate MPU9150 driver with TWI instance handler
    ret_code = mpu9150_init(&m_twi_instance);
    APP_ERROR_CHECK(ret_code); // Check for errors in return value
    
    // Setup and configure the MPU9150 with intial values
    mpu9150_config_t p_mpu_config = MPU9150_DEFAULT_CONFIG(); // Load default values
    p_mpu_config.smplrt_div = 19;   // Change sampelrate. Sample Rate = Gyroscope Output Rate / (1 + SMPLRT_DIV). 19 gives a sample rate of 50Hz
    p_mpu_config.accel_config.afs_sel = AFS_2G; // Set accelerometer full scale range to 2G
    ret_code = mpu9150_config(&p_mpu_config); // Configure the MPU9150 with above values
    APP_ERROR_CHECK(ret_code); // Check for errors in return value
    
    
    // This is a way to configure the interrupt pin behaviour
    // NOT YET IMPLEMENTED IN THIS EXAMPLE. YOU CAN DO IT YOURSELF BUT YOU WILL HAVE TO SET UP THE NRF51 INTERRUPT PINS AND IRQ ROUTINES YOURSELF
//    mpu9150_int_pin_cfg_t p_int_pin_cfg = MPU9150_DEFAULT_INT_PIN_CONFIG(); // Default configurations
//    p_int_pin_cfg.int_rd_clear = 1; // When this bit is equal to 1, interrupt status bits are cleared on any read operation
//    ret_code = mpu9150_int_cfg_pin(&p_int_pin_cfg); // Configure pin behaviour
//    APP_ERROR_CHECK(ret_code); // Check for errors in return value
//    
//    // Enable the MPU interrupts
//    mpu9150_int_enable_t p_int_enable = MPU9150_DEFAULT_INT_ENABLE_CONFIG();
//    p_int_enable.data_rdy_en = 1; // Trigger interrupt everytime new sensor values are available
//    ret_code = mpu9150_int_enable(&p_int_enable); // Configure interrupts
//    APP_ERROR_CHECK(ret_code); // Check for errors in return value    
}



/**
 *  The begin of the journey
 */
int main(void)
{
	
	ret_code_t err_code;

	const app_uart_comm_params_t comm_params =
    {
        RX_PIN_NUMBER,
        TX_PIN_NUMBER,
        RTS_PIN_NUMBER,
        CTS_PIN_NUMBER,
        APP_UART_FLOW_CONTROL_ENABLED,
        false,
        UART_BAUDRATE_BAUDRATE_Baud38400
    };

    APP_UART_FIFO_INIT(&comm_params,
                       UART_RX_BUF_SIZE,
                       UART_TX_BUF_SIZE,
                       uart_error_handle,
                       APP_IRQ_PRIORITY_LOW,
                       err_code);

    APP_ERROR_CHECK(err_code);
		
		
    twi_init();
    mpu_init();
        
    accel_values_t acc_values;
    uint32_t sample_number = 0;
	
    /* Initialization of UART */
    LEDS_CONFIGURE(LEDS_MASK);
    LEDS_OFF(LEDS_MASK);

    nrf_delay_ms(10000);
		
		
	printf("hello!\r\n");
	nrf_delay_ms(1000);
	  
    printf("MPU9150 example. Compiled @ %s\r\n", __TIME__); //\033[2J\033[;H
		nrf_delay_ms(1000);
	
    /* Main loop */
    while(1)
    {
				// Read accelerometer sensor values
        err_code = mpu9150_read_accel(&acc_values);
        APP_ERROR_CHECK(err_code);
        // Clear terminal and print values
        printf("\033[2J\033[;HSample # %d\r\nX: %06d\r\nY: %06d\r\nZ: %06d", ++sample_number, acc_values.x, acc_values.y, acc_values.z);
        nrf_delay_ms(250);
        //
      
    }
}

/** @} */ /* End of group twi_master_with_twis_slave_example */
